/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
    i2c-dev.h - i2c-bus driver, char device interface

    Copyright (C) 1995-97 Simon G. Vogl
    Copyright (C) 1998-99 Frodo Looijaard <frodol@dds.nl>

*/
#ifndef _LINUX_I2C_DEV_H
#define _LINUX_I2C_DEV_H

#include <uapi/linux/i2c-dev.h>

#define I2C_MAJOR	89		/* Device major number		*/
#endif /* _LINUX_I2C_DEV_H */
